import type { CalendarType, RangeType } from './types';
export declare function getDayOfWeek(date: Date, calendarType?: CalendarType): number;
/**
 * Century
 */
export declare function getBeginOfCenturyYear(date: Date): number;
/**
 * Decade
 */
export declare function getBeginOfDecadeYear(date: Date): number;
/**
 * Week
 */
/**
 * Returns the beginning of a given week.
 *
 * @param {Date} date Date.
 * @param {string} [calendarType="ISO 8601"] Calendar type.
 */
export declare function getBeginOfWeek(date: Date, calendarType?: CalendarType): Date;
/**
 * Gets week number according to ISO 8601 or US standard.
 * In ISO 8601, Arabic and Hebrew week 1 is the one with January 4.
 * In US calendar week 1 is the one with January 1.
 *
 * @param {Date} date Date.
 * @param {string} [calendarType="ISO 8601"] Calendar type.
 */
export declare function getWeekNumber(date: Date, calendarType?: CalendarType): number;
/**
 * Others
 */
/**
 * Returns the beginning of a given range.
 *
 * @param {string} rangeType Range type (e.g. 'day')
 * @param {Date} date Date.
 */
export declare function getBegin(rangeType: RangeType, date: Date): Date;
export declare function getBeginPrevious(rangeType: RangeType, date: Date): Date;
export declare function getBeginNext(rangeType: RangeType, date: Date): Date;
export declare function getBeginPrevious2(rangeType: RangeType, date: Date): Date;
export declare function getBeginNext2(rangeType: RangeType, date: Date): Date;
/**
 * Returns the end of a given range.
 *
 * @param {string} rangeType Range type (e.g. 'day')
 * @param {Date} date Date.
 */
export declare function getEnd(rangeType: RangeType, date: Date): Date;
export declare function getEndPrevious(rangeType: RangeType, date: Date): Date;
export declare function getEndPrevious2(rangeType: RangeType, date: Date): Date;
/**
 * Returns an array with the beginning and the end of a given range.
 *
 * @param {string} rangeType Range type (e.g. 'day')
 * @param {Date} date Date.
 */
export declare function getRange(rangeType: RangeType, date: Date): [Date, Date];
/**
 * Creates a range out of two values, ensuring they are in order and covering entire period ranges.
 *
 * @param {string} rangeType Range type (e.g. 'day')
 * @param {Date} date1 First date.
 * @param {Date} date2 Second date.
 */
export declare function getValueRange(rangeType: RangeType, date1: Date, date2: Date): Date[];
/**
 * @callback FormatYear
 * @param {string} locale Locale.
 * @param {Date} date Date.
 * @returns {string} Formatted year.
 */
/**
 * Returns a string labelling a century of a given date.
 * For example, for 2017 it will return 2001-2100.
 *
 * @param {string} locale Locale.
 * @param {FormatYear} formatYear Function to format a year.
 * @param {Date|string|number} date Date or a year as a string or as a number.
 */
export declare function getCenturyLabel(locale: string | undefined, formatYear: (locale: string | undefined, date: Date) => string, date: Date): string;
/**
 * Returns a string labelling a century of a given date.
 * For example, for 2017 it will return 2011-2020.
 *
 * @param {string} locale Locale.
 * @param {FormatYear} formatYear Function to format a year.
 * @param {Date|string|number} date Date or a year as a string or as a number.
 */
export declare function getDecadeLabel(locale: string | undefined, formatYear: (locale: string | undefined, date: Date) => string, date: Date): string;
/**
 * Returns a boolean determining whether a given date is the current day of the week.
 *
 * @param {Date} date Date.
 */
export declare function isCurrentDayOfWeek(date: Date): boolean;
/**
 * Returns a boolean determining whether a given date is a weekend day.
 *
 * @param {Date} date Date.
 * @param {string} [calendarType="ISO 8601"] Calendar type.
 */
export declare function isWeekend(date: Date, calendarType?: CalendarType): boolean;
